//////////////////////////////////////////////////////////////////////
// Wave.h: interface for the CWave class.
//////////////////////////////////////////////////////////////////////
// Copyright 1999.    Moe Wheatley AE4JY  <ae4jy@mindspring.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WAVE_H__43BFB6C1_EBE9_11D1_B48C_444553540000__INCLUDED_)
#define AFX_WAVE_H__43BFB6C1_EBE9_11D1_B48C_444553540000__INCLUDED_

#include <mmsystem.h>
#include <mmreg.h>
#include "ErrorCodes.h"


#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


//m_ChainWaveFile  Bit defs need to be in 8 bit byte
#define FILE_CHAIN 1
#define FILE_CHAIN_TIMESTAMP 2
#define FILE_CHAIN_FREQSTAMP 4

#define SOURCE_UNKNOWN 0
#define SOURCE_SV 1
#define SOURCE_PERSEUS 2
#define SOURCE_WINRAD 3

//+++++++++++++   WAVEFORMATEX  member variables     +++++++++++++++++++
//typedef struct { 
//    WORD  wFormatTag; 
//    WORD  nChannels; 
//    DWORD nSamplesPerSec; 
//    DWORD nAvgBytesPerSec; 
//    WORD  nBlockAlign; 
//    WORD  wBitsPerSample; 
//    WORD  cbSize; 
//} WAVEFORMATEX; 

typedef	struct{				//starting with SV version 3.0 (earlier versions stored local SYSTEMTIME by mistake)
	SYSTEMTIME StartTime;
	SYSTEMTIME StopTime;
	DWORD CenterFreq;
	DWORD ADFrequency;
	DWORD IFFrequency;
	DWORD Bandwidth;
	DWORD IQOffset;
	DWORD DBOffset;
	DWORD Unused3;
	DWORD Unused4;
	DWORD Unused5;
	DWORD Unused6;
}AUXINFO;

typedef	struct{				//SV version 2.34 and earlier versions
	SYSTEMTIME StartTime;
	SYSTEMTIME StopTime;
	DWORD CenterFreq;
	DWORD ADFrequency;
	DWORD IFFrequency;
	DWORD Bandwidth;
	DWORD IQOffset;
	DWORD DBOffset;
	DWORD Unused3;
	DWORD Unused4;
	DWORD Unused5;
}AUX3INFO;

typedef	struct{				//SV real early probably no files exist
	SYSTEMTIME StartTime;
	SYSTEMTIME StopTime;
	DWORD CenterFreq;
	DWORD ADFrequency;
	DWORD Bandwidth;
}AUX2INFO;

typedef	struct{				//SV real early probably no files exist
	SYSTEMTIME StartTime;
	DWORD CenterFreq;
	DWORD ADFrequency;
	DWORD Bandwidth;
}AUX1INFO;

typedef	struct{				//Winrad uses same auxi tag but different data structure
	SYSTEMTIME StartTime;
	SYSTEMTIME StopTime;
	DWORD CenterFreq;
	DWORD Spare[32];
}AUXWINRAD;


typedef struct
{
//	char  chunkID[4];	// ="rcvr"
//	long  chunkSize;	// length of the chunk
	long nCenterFrequencyHz;	// Center frequency, expressed in Hz
	long SamplingRateIdx;		// 0=125K, 1=250K, 2=500K, 3=1Ms/s
	time_t timeStart;			// time of the start of the recording (as reported by time(&timeStart))
	unsigned short wAttenId;	// 0=0dB, 1=10dB, 2=20dB, 3=30dB attenuator value
	char bAdcPresel;	// 0=Presel Off, 1=Presel On
	char bAdcPreamp;	// 0=ADC Preamp Off, 1=ADC Preamp ON
	char bAdcDither;	// 0=ADC Dither Off, 1=ADC Dither ON
	char bSpare;		// for future use (default = 0)
	char rsrvd[16];		// for future use (default = 000..0)
 }tPerseusHdr;


class CWave  
{
public:
	void ChangeOutFileParameters( int IFFreq);
	CWave();
	virtual ~CWave();
	// Public member functions for object's users.
	LONGLONG GetDriveFreeSpace(CString* pPathName);
	UINT InOpen( CString* pPathName, WAVEFORMATEX* pWFX, DWORD BufSize,
						PDWORD pFileSize, DWORD Offset, AUXINFO* pAuxInfo, DWORD* pSource);
	LONG InRead( SHORT* pData , INT Length, DWORD& Progress  );
	LONG InRead( _int32* pData , INT Length, DWORD& Progress );
	BOOL ChangeFilePosition(INT position);
	time_t GetInFilePosition();
	DWORD GetInTotalSecs(){return m_FileTotalSecs;};
	DWORD GetInPositionSecs();
	void SetInPositionSecs(DWORD secs);
	void InClose();
	UINT OutOpen( CString* pPathName, WAVEFORMATEX* pWFX, DWORD BufSize,
					DWORD SampleLimit, INT ChainFiles, AUXINFO* pAuxInfo);
	LONG OutWrite( SHORT* pData , INT Length );
	LONG OutWrite( _int32* pData , INT Length );
	void UseOpenTimestamp();
	void OutClose();
	UINT GetError(void){return m_ErrorCode;}
	BOOL IsOutOpen(){ return m_OutputOpen;}
	BOOL IsInOpen(){ return m_InputOpen;}

	INT	m_ChainCount;

private:
	SYSTEMTIME ConvertLocalToUTC(SYSTEMTIME* pSlocal);
	INT m_BufferSize;
	BOOL m_UseAuxStructure;
	CString m_OutPath;
	DWORD m_AuxPosition;
	DWORD m_BytesPerSecond;
	DWORD m_DataStartPos;
	DWORD m_FileBlockSize;
	DWORD m_FileTotalSecs;
	BOOL m_ContLoopMode;
	BOOL m_InputOpen;
	BOOL m_OutputOpen;
	INT m_ChainFiles;
	INT m_InBytesPerSample;
	INT m_OutBytesPerSample;
	DWORD m_InBlocksRead;
	UINT m_ErrorCode;
	LONG m_InBufferSize;		// internal byte buffer
	LONG m_OutBufferSize;		// internal byte buffer
	LONG m_InBufPosition;		// current byte inbuffer position
	LONG m_InBufferEndPos;
	LONG m_OutBufPosition;		// current byte outbuffer position
	DWORD m_OutSampleLimit;
	DWORD m_OutSamplesWritten;
	CString m_Str;
	HMMIO m_INhmmio;
	HMMIO m_OUThmmio;
	SYSTEMTIME m_FileStartTime;
	CTime m_StartCtime;
	CTime m_StopCtime;
	AUXINFO m_OutAuxInfo;
	WAVEFORMATEX m_InFormat;
	WAVEFORMATEX m_OutFormat;
	MMCKINFO m_inmmcktSubchunk;
	MMCKINFO m_inmmcktRIFF;
	MMCKINFO m_outmmcktSubchunk;
	MMCKINFO m_outmmcktRIFF;

	tPerseusHdr m_PerseusHdr;
	CRITICAL_SECTION m_CriticalSection;	// use for keeping threads
										// from stomping on each other
	char * m_pInBuffer;
	char * m_pOutBuffer;
	union uSWORD{
		struct{
			BYTE lsb;
			BYTE msb;
		}bytes;
		SHORT both;
	}m_usTemp;
	union uFLOAT{
		struct{
			BYTE b0;
			BYTE b1;
			BYTE b2;
			BYTE b3;
		}bytes;
		float all;
	}m_ufTemp;
	union uINT32{
		struct{
			BYTE b0;
			BYTE b1;
			BYTE b2;
			BYTE b3;
		}bytes;
		__int32 all;
	}m_uiTemp;
};

#endif // !defined(AFX_WAVE_H__43BFB6C1_EBE9_11D1_B48C_444553540000__INCLUDED_)
