// WaveVueView.cpp : implementation of the CWaveVueView class
//

#include "stdafx.h"
#include "WaveVue.h"

#include "WaveVueDoc.h"
#include "WaveVueView.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWaveVueView

IMPLEMENT_DYNCREATE(CWaveVueView, CResizableFormView)

BEGIN_MESSAGE_MAP(CWaveVueView, CResizableFormView)
	//{{AFX_MSG_MAP(CWaveVueView)
	ON_BN_CLICKED(IDC_SRCBUTTON, OnSrcbutton)
	ON_BN_CLICKED(IDC_SAVEBUTTON, OnSavebutton)
	ON_BN_CLICKED(IDC_QUITBUTTON, OnQuitbutton)
	ON_NOTIFY(UDN_DELTAPOS, IDC_FFTSPIN, OnDeltaposFftspin)
	ON_BN_CLICKED(IDC_DESTBUTTON, OnDestbutton)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_AGCCHECK, OnAgccheck)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWaveVueView construction/destruction

CWaveVueView::CWaveVueView()
	: CResizableFormView(CWaveVueView::IDD)
{
	//{{AFX_DATA_INIT(CWaveVueView)
	m_DestPathStr = _T("");
	m_SrcPathStr = _T("");
	m_FileStartTimeStr = _T("");
	m_FileEndTimeStr = _T("");
	m_FromTimeStr = _T("");
	m_ToTimeStr = _T("");
	m_UseAGC = FALSE;
	m_OffsetTxt = _T("");
	//}}AFX_DATA_INIT
	// TODO: add construction code here
	m_pPlotData = NULL;
}

CWaveVueView::~CWaveVueView()
{
}

void CWaveVueView::DoDataExchange(CDataExchange* pDX)
{
	CResizableFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CWaveVueView)
	DDX_Control(pDX, IDC_CURSORTIME, m_CursotTimeCtrl);
	DDX_Control(pDX, IDC_SAVEBUTTON, m_SaveCtrl);
	DDX_Text(pDX, IDC_DESTTXT, m_DestPathStr);
	DDX_Text(pDX, IDC_SRCTXT, m_SrcPathStr);
	DDX_Text(pDX, IDC_STARTTIME, m_FileStartTimeStr);
	DDX_Text(pDX, IDC_ENDTIME, m_FileEndTimeStr);
	DDX_Text(pDX, IDC_FROMTIMETXT, m_FromTimeStr);
	DDX_Text(pDX, IDC_TOTIMETXT, m_ToTimeStr);
	DDX_Check(pDX, IDC_AGCCHECK, m_UseAGC);
	DDX_Text(pDX, IDC_OFSETTXT, m_OffsetTxt);
	//}}AFX_DATA_MAP
  	DDX_Control(pDX, IDC_FILESCROLLCTRL, m_WposCtrl);
}

BOOL CWaveVueView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CResizableFormView::PreCreateWindow(cs);
}

void CWaveVueView::OnInitialUpdate()
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
	AddAnchor(IDC_PLOTAREA, TOP_LEFT, BOTTOM_RIGHT);
	AddAnchor(IDC_FILESCROLLCTRL, BOTTOM_LEFT, BOTTOM_RIGHT);
	AddAnchor(IDC_HELPTXT, BOTTOM_LEFT, BOTTOM_RIGHT);
	

	AddAnchor(IDC_SRCBUTTON, TOP_LEFT, NOANCHOR);
	AddAnchor(IDC_DESTBUTTON, TOP_LEFT, NOANCHOR);
	AddAnchor(IDC_FFTSPIN, MIDDLE_LEFT, NOANCHOR);

	AddAnchor(IDC_SRCTXT, TOP_LEFT, TOP_RIGHT);
	AddAnchor(IDC_DESTTXT, TOP_LEFT, TOP_RIGHT);

	AddAnchor(IDC_STARTTIME, BOTTOM_LEFT, NOANCHOR);
	AddAnchor(IDC_SAVEBUTTON, BOTTOM_LEFT, NOANCHOR);
	AddAnchor(IDC_AGCCHECK, BOTTOM_LEFT, NOANCHOR);
	AddAnchor(IDC_OFSETTXT, BOTTOM_LEFT, NOANCHOR);
	
	AddAnchor(IDC_ENDTIME, BOTTOM_RIGHT, NOANCHOR);
	AddAnchor(IDC_QUITBUTTON, BOTTOM_RIGHT, NOANCHOR);

	AddAnchor(IDC_FROMTIMETXT, BOTTOM_CENTER, NOANCHOR);
	AddAnchor(IDC_TOTIMETXT, BOTTOM_CENTER, NOANCHOR);
	AddAnchor(IDC_FROMLBL, BOTTOM_CENTER, NOANCHOR);
	AddAnchor(IDC_TOLBL, BOTTOM_CENTER, NOANCHOR);
	AddAnchor(IDC_CURSORTIME, BOTTOM_CENTER, NOANCHOR);

	AddAnchor(IDC_SAVEBUTTON, BOTTOM_LEFT, NOANCHOR);

	CResizableFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();
	m_WposCtrl.SetCallback( this, (PCALLBACKFUNCT)WavePosCallback);
//
	if(!m_pPlotData)
		m_pPlotData = new CPlotData(pDoc);
	m_SaveCtrl.EnableWindow(FALSE);
	m_SrcPathStr = pDoc->m_SrcFilePath;
	m_DestPathStr = pDoc->m_DestFilePath;
	StartFileRead(pDoc->m_SrcFilePath);
	
	m_OffsetTxt.Format("%d",pDoc->m_FFTOffset);
	UpdateData(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CWaveVueView diagnostics

#ifdef _DEBUG
void CWaveVueView::AssertValid() const
{
	CFormView::AssertValid();
}

void CWaveVueView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CWaveVueDoc* CWaveVueView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CWaveVueDoc)));
	return (CWaveVueDoc*)m_pDocument;
}
#endif //_DEBUG



/////////////////////////////////////////////////////////////////////////////
// CWaveVueView message handlers

void CWaveVueView::OnDestroy() 
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
	CResizableFormView::OnDestroy();
	pDoc->CloseWaveFile();
	if(m_pPlotData)
		delete m_pPlotData;
	
}

void CWaveVueView::InvalidateCursor( void )
{
   POINT pt; // Screen coordinates!
   GetCursorPos( &pt );
   SetCursorPos( pt.x, pt.y );
}

void CWaveVueView::OnDraw(CDC* pDC) 
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
	//go recalculate & draw the data plots
	if(m_pPlotData)
	{
		// Set hourglass cursor:
		SetCursor( LoadCursor( 0, IDC_WAIT ) );
		m_pPlotData->OnDrawPlot( this, pDC );
		// Restore normal cursor:
		InvalidateCursor();
		m_WposCtrl.SetSegStopSecs(pDoc->m_SegEndPosition);
	}
}

void CWaveVueView::DrawPlot()
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
	CClientDC dc(this);
	if(m_pPlotData && pDoc->m_WaveFileOpen)
	{
	// Set hourglass cursor:
		SetCursor( LoadCursor( 0, IDC_WAIT ) );
		m_pPlotData->DrawPlot(&dc);
		m_WposCtrl.SetSegStopSecs(pDoc->m_SegEndPosition);
		// Restore normal cursor:
		InvalidateCursor();
	}
}

void CWaveVueView::WavePosCallback( INT type, INT secs)
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
int segwidth = m_WposCtrl.GetSegStopSecs() - m_WposCtrl.GetSegStartSecs();
	if(pDoc->m_Wave.IsInOpen() )
	{
		switch(type)
		{
			case MSG_TYPE_SEGMENT:
				if(secs<0)
					secs = 0;
				if(secs > ( (int)pDoc->m_TotalSeconds - segwidth) )
					secs = (pDoc->m_TotalSeconds - segwidth);
				pDoc->m_CurrentFilePosition = secs;
				pDoc->m_Wave.SetInPositionSecs(secs);
				m_WposCtrl.SetSegStartSecs(secs);
				DrawPlot();
				m_Str.Format("View %d Sec to %d Sec", m_WposCtrl.GetSegStartSecs(), m_WposCtrl.GetSegStopSecs() );
				m_CursotTimeCtrl.SetWindowText(m_Str);
				break;
			case MSG_TYPE_LMKR:
				m_WposCtrl.SetLMkrSecs(secs);
				UpdateToFrom();
				break;
			case MSG_TYPE_RMKR:
				m_WposCtrl.SetRMkrSecs(secs);
				UpdateToFrom();
				break;
		}
	}
}

void CWaveVueView::OnSrcbutton() 
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
//  handler code here for getting Input filename
CString sFilter = "Wave Files (*.wav)|*.wav||";
CFileDialog Dlgfile( TRUE, "wav", pDoc->m_SrcFilePath, 
			OFN_HIDEREADONLY, (LPCTSTR)sFilter, this );
	m_SaveCtrl.EnableWindow(FALSE);
	pDoc->CloseWaveFile();
	if( Dlgfile.DoModal() == IDOK)
	{
		if(pDoc->GetFileParameters(Dlgfile.GetPathName()) )
			pDoc->m_SrcFilePath = Dlgfile.GetPathName();
		m_SrcPathStr = pDoc->m_SrcFilePath;
		StartFileRead(m_SrcPathStr);
		DrawPlot();
	}
}


void CWaveVueView::OnQuitbutton() 
{
	AfxGetMainWnd()->PostMessage(WM_CLOSE);
}

void CWaveVueView::OnDeltaposFftspin(NMHDR* pNMHDR, LRESULT* pResult) 
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
	NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
	*pResult = 0;
	pDoc->m_FFTOffset = pDoc->m_FFTOffset + (pNMUpDown->iDelta*20);
	if(pDoc->m_FFTOffset>100 )
	{
		pDoc->m_FFTOffset = 100;
		return;
	}
	if(pDoc->m_FFTOffset<-100)
	{
		pDoc->m_FFTOffset = -100;
		return;
	}
	if(m_pPlotData)
		m_pPlotData->SetFFTParams( pDoc->m_Complex, (double)pDoc->m_FFTOffset);
	m_OffsetTxt.Format("%d",pDoc->m_FFTOffset);
	UpdateData(FALSE);
	pDoc->m_LastFilePosition = -1;
	DrawPlot();	
}

void CWaveVueView::OnDestbutton() 
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
CString sFilter = "Wave Files (*.wav)|*.wav||";
CString str;
CFileDialog DlgOutfile( FALSE, _T("wav"), pDoc->m_DestFilePath, 
			OFN_LONGNAMES|OFN_NOCHANGEDIR|OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT,
			(LPCTSTR)sFilter, this );
	if( DlgOutfile.DoModal() == IDOK)
	{
		pDoc->m_DestFilePath = DlgOutfile.GetPathName();
		m_DestPathStr = pDoc->m_DestFilePath;
		if(m_SrcPathStr!="")
			m_SaveCtrl.EnableWindow(TRUE);
	}	
	UpdateData(FALSE);
}

void CWaveVueView::StartFileRead(CString path)
{
CTime time;
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
	if( pDoc->OpenWaveFile(path) )
	{
		m_WposCtrl.SetTotalSecs(pDoc->m_TotalSeconds);
		m_WposCtrl.SetRMkrSecs(pDoc->m_TotalSeconds);
		pDoc->m_ToSeconds = pDoc->m_TotalSeconds;

		m_WposCtrl.SetSegStartSecs(0);
		pDoc->m_FromSeconds = 0;
		m_WposCtrl.SetLMkrSecs(0);
		pDoc->m_CurrentFilePosition = 0;
		pDoc->m_LastFilePosition = -1;

		pDoc->m_WaveFileOpen = TRUE;
		if(m_DestPathStr!="")
			m_SaveCtrl.EnableWindow(TRUE);
		if(m_pPlotData)
			m_pPlotData->SetFFTParams( pDoc->m_Complex, pDoc->m_FFTOffset);
		m_Str.Format("(%d Sec)", pDoc->m_TotalSeconds);
		if(pDoc->m_StartTime.wYear>2000)
		{
			time = pDoc->m_StartTime;
			m_FileStartTimeStr = time.Format(_T("%m/%d/%Y %#H:%M:%S (0 Sec)"));
			if(pDoc->m_EndTime.wYear>2000)
			{
				time = pDoc->m_EndTime;
				m_FileEndTimeStr = time.Format(_T("%m/%d/%Y %#H:%M:%S "));
				m_FileEndTimeStr += m_Str;
			}
			else
				m_FileEndTimeStr = m_Str;
		}
		else
		{
			m_FileStartTimeStr = "0 Sec";
			m_FileEndTimeStr = m_Str;
		}
		UpdateToFrom();
	}
	else
		pDoc->m_WaveFileOpen = FALSE;
}



void CWaveVueView::UpdateToFrom()
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
CTime time(pDoc->m_StartTime);
time_t ft = (time_t)m_WposCtrl.GetLMkrSecs();
time_t tt = (time_t)m_WposCtrl.GetRMkrSecs();

	if(pDoc->m_StartTime.wYear>2000)
	{
		m_Str.Format("(%d Secs)",ft);
		ft += time.GetTime();
		pDoc->m_ftime = ft;
		m_FromTimeStr = pDoc->m_ftime.Format(_T("%m/%d/%Y %#H:%M:%S "));
		m_FromTimeStr += m_Str;
		m_Str.Format("(%d Secs)",tt);
		tt += time.GetTime();
		pDoc->m_ttime = tt;
		m_ToTimeStr = pDoc->m_ttime.Format(_T("%m/%d/%Y %#H:%M:%S "));
		m_ToTimeStr += m_Str;
	}
	else
	{
		m_FromTimeStr.Format("%d Secs",ft);
		pDoc->m_ftime = ft;
		m_ToTimeStr.Format("%d Secs",tt);
		pDoc->m_ttime = tt;
	}
	UpdateData(FALSE);

}

void CWaveVueView::OnSavebutton() 
{
INT i;
double sampI,sampQ;
double abssamp;
INT ErrorCode;
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
CFile file;	
	if( file.Open(pDoc->m_DestFilePath,CFile::modeRead,NULL) )
	{
		file.Close();
		if(AfxMessageBox("File exists.  Overwrite?",MB_YESNO, 0) ==IDNO)
		{
			return;
		}
	}
	//Here if ok to write file segment
#if 0
	m_IQOffset = (double)pDoc->m_AuxInfo.IQOffset/ 1000.0;
	pDoc->m_AuxInfo.IQOffset = 0;
#endif
	pDoc->m_ftime.GetAsSystemTime(pDoc->m_AuxInfo.StartTime);
	pDoc->m_ttime.GetAsSystemTime(pDoc->m_AuxInfo.StopTime);
	ErrorCode = pDoc->m_Wave.OutOpen( &pDoc->m_DestFilePath,
									&pDoc->m_Wfx,
									FFTSIZE,
									0,
									TRUE,
									&pDoc->m_AuxInfo );
	if(ErrorCode)
		return;

	// Set hourglass cursor:
	SetCursor( LoadCursor( 0, IDC_WAIT ) );
	INT SamplesToWrite;
	DWORD prog;
	LONG sampproc;
	SamplesToWrite = (INT)(((LONGLONG)pDoc->m_FileLength*((LONGLONG)m_WposCtrl.GetRMkrSecs() - (LONGLONG)m_WposCtrl.GetLMkrSecs() ))/(LONGLONG)pDoc->m_TotalSeconds);
	pDoc->m_Wave.SetInPositionSecs(m_WposCtrl.GetLMkrSecs() );
	m_IQAve = 0;
	m_t	= -0.0;
	m_tsec= -66.0;
	while( SamplesToWrite > 0)
	{
		sampproc = pDoc->m_Wave.InRead(m_buf,FFTSIZE,prog);
		if(sampproc!=FFTSIZE)
			break;
		if( m_UseAGC )
		{
#if 0
			DeDoppler(m_buf);
#else
			for(i=0; i<FFTSIZE*2; i+=2)
			{
				sampI = (double)m_buf[i];
				sampQ = (double)m_buf[i+1];
				abssamp = fabs(sampI);
				if(abssamp>m_IQAve)
					m_IQAve = abssamp;
				else
					m_IQAve = (1.0-1.0/100.0)*m_IQAve + (1.0/100.0)*abssamp;
				if(m_IQAve > 0.0001)
				{
					sampI = sampI/m_IQAve;
					sampQ = sampQ/m_IQAve;
					m_buf[i] = (SHORT)(sampI*200.);
					m_buf[i+1] = (SHORT)(sampQ*200.);
				}
			}
#endif
		}
		sampproc = pDoc->m_Wave.OutWrite(m_buf,FFTSIZE);
		if(sampproc!=FFTSIZE)
			break;
		SamplesToWrite -= FFTSIZE;
	}
	pDoc->m_Wave.UseOpenTimestamp();
	pDoc->m_Wave.OutClose();
	pDoc->m_Wave.SetInPositionSecs(pDoc->m_CurrentFilePosition);
	// Restore normal cursor:
	InvalidateCursor();
}

void CWaveVueView::OnAgccheck() 
{
	UpdateData(TRUE);	
}

void CWaveVueView::DeDoppler(SHORT *pBuf)
{
CWaveVueDoc* pDoc = GetDocument();
	ASSERT_VALID( pDoc );
#define PI2 ( 8.0 * atan(1.0) )		// 2 Pi
#define SAMPRATE 16276.
#define NCOINC (PI2/SAMPRATE)
#define KA 4138.0
#define KP 7061.0
#define KPS (PI2/KP)
#define KO .91
#define KS (-.25/60.0)
#define KD (160.0+0)
double t;
double tsec;
double cor;
_complex acc;
_complex offset;
_complex samp;
double IQOffset = m_IQOffset;
	t = m_t;
	tsec = m_tsec;
	for(int i=0; i<FFTSIZE*2; i+=2)
	{
		tsec = tsec + 1.0/SAMPRATE;
		cor = KA*sin(KPS*tsec + KO) + KS*tsec + KD;
		offset.y = cos(t);
		offset.x = sin(t);
		acc.y = (double)pBuf[i] + IQOffset;
		acc.x = (double)pBuf[i+1] + IQOffset;
		samp.x = ((offset.x*acc.x) - (offset.y*acc.y));
		samp.y = ((offset.x*acc.y) + (offset.y*acc.x));
		t = t - NCOINC*cor;
		pBuf[i] = (SHORT)(samp.x);
		pBuf[i+1] = (SHORT)(samp.y);
	}
	t = fmod(t,PI2);	//keep radian counter bounded
	m_t = t;
	m_tsec = tsec;
}

void CWaveVueView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if( m_pPlotData->CurInPlot(point) )
	{	//if in display area and cursor position is to be displayed
		int sec = m_pPlotData->GetCursorSecs(m_WposCtrl.GetSegStartSecs(),
									m_WposCtrl.GetSegStopSecs(), point.x);
		m_Str.Format("Cursor = %d Sec", sec);;
	}	
	else
	{
		m_Str.Format("View %d Sec to %d Sec", m_WposCtrl.GetSegStartSecs(), m_WposCtrl.GetSegStopSecs() );
	}
	m_CursotTimeCtrl.SetWindowText(m_Str);
	CResizableFormView::OnMouseMove(nFlags, point);
}

void CWaveVueView::OnLButtonDown(UINT nFlags, CPoint point) 
{
CWaveVueDoc* pDoc = GetDocument();
	if( m_pPlotData->CurInPlot(point) )
	{	//if in display area
		int segwidth = m_WposCtrl.GetSegStopSecs() - m_WposCtrl.GetSegStartSecs();
		int sec = m_pPlotData->GetCursorSecs(m_WposCtrl.GetSegStartSecs(),
									m_WposCtrl.GetSegStopSecs(), point.x);
		int newstart = sec - segwidth/2;
		if(newstart<0)
			newstart = 0;
		if(newstart > ((int)pDoc->m_TotalSeconds - segwidth) )
			newstart = (pDoc->m_TotalSeconds - segwidth);
		WavePosCallback( MSG_TYPE_SEGMENT, newstart);
	}
	CResizableFormView::OnLButtonDown(nFlags, point);
}
